from ariadne import gql

from bot_data.abc.query_abc import QueryABC
from bot_data.graphql.types.client_query_type import ClientQueryType
from bot_data.graphql.types.level_query_type import LevelQueryType
from bot_data.graphql.types.user_query_type import UserQueryType
from bot_data.model.server import Server


class ServerQueryType(QueryABC):
    __schema__ = gql("""
        type Server {
            id: ID
            discord_id: String
            clients: [Client]
            users: [User]
            levels: [Level]
        }
    """)
    __requires__ = [ClientQueryType, UserQueryType, LevelQueryType]

    @classmethod
    def resolve_id(cls, server: Server, *_):
        return server.server_id

    @classmethod
    def resolve_discord_id(cls, server: Server, *_):
        return server.discord_server_id

    @classmethod
    def resolve_clients(cls, server: Server, *_):
        return cls._clients.where(lambda c: c.server.server_id == server.server_id)

    @classmethod
    def resolve_users(cls, server: Server, *_):
        return cls._users.where(lambda u: u.server.server_id == server.server_id)

    @classmethod
    def resolve_levels(cls, server: Server, *_):
        return cls._levels.where(lambda l: l.server.server_id == server.server_id)
