from ariadne import gql

from bot_data.abc.query_abc import QueryABC
from bot_data.graphql.types.level_query_type import LevelQueryType
from bot_data.model.user import User


class UserQueryType(QueryABC):
    __schema__ = gql("""
        type User {
            id: ID
            discord_id: String
            name: String
            xp: Int
            ontime: Int
            level: Level
        }
    """)
    __requires__ = [LevelQueryType]

    @classmethod
    def resolve_id(cls, user: User, *_):
        return user.user_id

    @classmethod
    def resolve_name(cls, user: User, *_):
        return cls._bot.get_user(user.discord_id).name

    @classmethod
    def resolve_ontime(cls, user: User, *_):
        return cls._user_joined_voice_channel.where(lambda ujvc: ujvc.user.user_id == user.user_id) \
            .where(lambda x: x.leaved_on is not None and x.joined_on is not None) \
            .sum(lambda join: round((join.leaved_on - join.joined_on).total_seconds() / 3600, 2))

    @classmethod
    def resolve_level(cls, user: User, *_):
        levels = cls._levels.where(lambda l: l.server.server_id == user.server.server_id).order_by(lambda l: l.min_xp)
        return levels.where(lambda l: user.xp >= l.min_xp).last_or_default()
    